package cn.edu.hfuu.iao.dc.battleship;

import java.io.IOException;

import javax.swing.JOptionPane;

/**
 * The communicator. Here you can implement all the code necessary for
 * performing the communication between the players.
 */
public class Communicator implements IBattleshipModelListener, Runnable {

  /** the model */
  private final BattleshipModel m_model;

  /** are we running? */
  private boolean m_running;

  // TODO
  // add your member variables here
  // TODO

  /**
   * Create the communicator
   *
   * @param m
   *          the model
   */
  public Communicator(final BattleshipModel m) {
    super();
    this.m_model = m;
  }

  /**
   * Setup the communicator
   *
   * @param ownPort
   *          the own port
   * @param enemyHost
   *          the enemy host
   * @param enemyPort
   *          the enemy port
   * @throws IOException
   *           if something goes wrong
   */
  public synchronized final void start(final int ownPort,
      final String enemyHost, final int enemyPort) throws IOException {

    if (!(this.m_running)) {

      // TODO
      // Add your startup code here, e.g., create sockets.
      // TODO

      this.m_model.addListener(this);
      this.m_running = true;

      new Thread(this).start();
    }
  }

  /**
   * Stop this thread
   */
  public synchronized final void stop() {
    System.out.println("Communicator has been stopped."); //$NON-NLS-1$
    if (this.m_running) {
      this.m_running = false;

      // TODO
      // add your shutdown code here (may not be called due to System.exit in
      // the event handler of the main frame, but insert it anyway ^_^)
      // TODO

    }
    System.out.println("Server resources released."); //$NON-NLS-1$
  }

  /**
   * The server loop
   */
  @Override
  public final void run() {
    while (this.m_running) {
      // TODO
      // Add your message processing code here.
      // This code should perform the server-side actions of receiving and
      // processing messages. This involves calling the methods
      // enemyHasSeen, enemyShipDiscovered, enemyIsReady.
      // TODO
    }
  }

  /** <p>
   * The player has clicked an enemy cell and wants to see whats
   * behind it. We need to tell "the other side", i.e., the enemy,
   * about that. In other words, we need to send a message with the
   * two coordinates x and y to the enemy. The "other side" will
   * then call the corresponding method {@link #enemyHasSeen(int, int)}.
   * </p><p>
   * If we have revealed a ship, the enemy's game engine will invoke
   * the method "playerShipDiscovered" on his side, which will then
   * send a message back to us.</p>
   *
   * @param x
   *          the x-coordinate of the cell
   * @param y
   *          the y-coordinate of the cell
   */
  public synchronized void playerHasSeen(final int x, final int y) {
    // TODO
    // send a message with the given data to the other side
    // TODO
  }

  /**
   * The enemy has discovered one of our ships. We should tell him that. The
   * other side will then call the method {@link #enemyShipDiscovered(int, int)}
   * .
   *
   * @param x
   *          the x-coordinate of the cell
   * @param y
   *          the y-coordinate of the cell
   */
  public synchronized void playerShipDiscovered(final int x, final int y) {
    // TODO
    // send a message with the given data to the other side
    // TODO
  }

  /**
   * The player is ready. We need to tell that to the other side. The other side
   * will then call the method {@link #enemyIsReady()}
   */
  public synchronized void playerIsReady() {//
    // TODO
    // send a message with the given data to the other side
    // TODO
  }

  /**
   * The enemy has seen a cell. The other side must have told us that.
   *
   * @param x
   *          the x-coordinate of the cell
   * @param y
   *          the y-coordinate of the cell
   */
  public synchronized final void enemyHasSeen(final int x, final int y) {
    try {
      this.m_model.enemyHasSeen(x, y);
    } catch (Throwable t) {
      this.onError(t);
    }
  }

  /**
   * We have discovered an enemy ship. The other side must have told us that.
   *
   * @param x
   *          the x-coordinate of the cell
   * @param y
   *          the y-coordinate of the cell
   */
  public synchronized final void enemyShipDiscovered(final int x,
      final int y) {
    try {
      this.m_model.enemyHasShip(x, y);
    } catch (Throwable t) {
      this.onError(t);
    }
  }

  /**
   * The enemy is ready. The other side must have told us that
   */
  public synchronized final void enemyIsReady() {
    try {
      this.m_model.enemyIsReady();
    } catch (Throwable t) {
      this.onError(t);
    }
  }

  //
  // better not touch stuff below that line
  //

  /**
   * An error has been caused. Maybe the game state was inconsistent. We should
   * terminate the game and maybe send an error message to the other side.
   *
   * @param t
   *          the error
   */
  public synchronized final void onError(final Throwable t) {
    this.stop();
    JOptionPane
        .showMessageDialog(
            null,
            "An error has occured that lead to game termination. The error message is '" //$NON-NLS-1$
                + t.getMessage() + "'.");//$NON-NLS-1$
    this.m_model.endGame(BattleshipModel.WINNER_NOBODY);
  }

  /** {@inheritDoc} */
  @Override
  public synchronized final void battleshipModelChanged(
      final BattleshipModelEvent event) {
    int whatHasChanged, oldState, change, state, /* winner, */x, y;
    final BattleshipModel model;

    whatHasChanged = event.whatHasChanged();
    oldState = event.getOldState();
    model = this.m_model;
    if ((whatHasChanged & BattleshipModelEvent.CHANGE_FLAG_GAME_STATE) != 0) {
      synchronized (model) {
        change = ((model.getGameState()) & (~oldState));
      }

      if ((change & BattleshipModel.GAME_STATE_PLAYER_READY) != 0) {
        this.playerIsReady();
      } else {
        if (((change & BattleshipModel.GAME_STATE_PLAYING) != 0)
            && ((oldState & BattleshipModel.GAME_STATE_PLAYER_READY) == 0)) {
          this.playerIsReady();
        }
      }

      return;
    }

    if ((whatHasChanged & BattleshipModelEvent.CHANGE_FLAG_CELL_STATE) != 0) {
      x = event.getX();
      y = event.getY();

      state = model.getCellState(x, y);
      change = (state & (~oldState));

      if ((change & BattleshipModel.CELL_STATE_PLAYER_HAS_SEEN) != 0) {
        this.playerHasSeen(x, y);
      }

      if (((change & BattleshipModel.CELL_STATE_ENEMY_HAS_SEEN) != 0)
          && ((state & BattleshipModel.CELL_STATE_PLAYER_SHIP) != 0)) {
        this.playerShipDiscovered(x, y);
      }
    }
  }
}
